
/*
 * Only works if run on the same host as postgres; can't work over a
 * network because we need to be able to signal backend directly.
 */

#include <stdlib.h>
#include <stdio.h>
#include <libpq-fe.h>
#include <sys/types.h>
#include <signal.h>

#define ERRBUF_SIZE 256

int main(int argc, char *argv[])
{
	char		*conninfo;
	PGconn		*conn;
	PGcancel	*cancel;
	char		 errbuf[ERRBUF_SIZE];
	pid_t		 be_pid;

	if (argc < 2)
	{
		fprintf(stderr, "must supply connection string as argument");
		exit(1);
	}

	conninfo = argv[1];

	conn = PQconnectdb(conninfo);

	/* Check to see that the backend connection was successfully made */
	if (PQstatus(conn) != CONNECTION_OK)
	{
		fprintf(stderr, "Connection to database failed: %s",
				PQerrorMessage(conn));
		PQfinish(conn);
		exit(1);
	}

	cancel = PQgetCancel(conn);

	be_pid = PQbackendPID(conn);

	printf("sending SIGSTOP to %d\n", be_pid);
	kill(be_pid, SIGSTOP);

	printf("sending query\n");
	PQsendQuery(conn, "select pg_sleep(1000)");

	printf("sending cancel\n");
	PQcancel(cancel, errbuf, ERRBUF_SIZE);

	printf("sending SIGCONT to %d\n", be_pid);
	kill(be_pid, SIGCONT);

	printf("disconnecting\n");
	PQfreeCancel(cancel);

	PQfinish(conn);

	return 0;
}
